const wiz_chatinput = document.getElementById("wiz-chatinput");
let is_sending_message = false;

async function sendChatMessage() {
    const message = wiz_chatinput.value.trim();
    if (!message || is_sending_message) return;
    
    is_sending_message = true;
    let chat_messages = [];
    
    try {
        // Get current chat messages and UUID in parallel
        const [wiz_chat, wiz_settings] = await Promise.all([
            chrome.storage.local.get('wiz_chat'),
            chrome.storage.local.get('wiz_settings')
        ]);
        
        chat_messages = wiz_chat.wiz_chat || [];
        const uuid = wiz_settings.wiz_settings?.extension_uuid || '';
        
        // Add user message and clear input
        chat_messages.push({ role: 'user', content: message });
        wiz_chatinput.value = '';
        
        // Update storage and display
        await chrome.storage.local.set({ wiz_chat: chat_messages });

        // Add loading message
        chat_messages.push({
            role: 'assistant',
            content: 'Thinking...'
        });
        await chrome.storage.local.set({ wiz_chat: chat_messages });
        loadChatMessages();

        // Send to API
        const response = await fetch(API_ENDPOINT + '/chat', {
            method: 'POST',
            headers: {
                'Content-Type': 'application/json',
                'Extension-UUID': uuid
            },
            body: JSON.stringify(chat_messages.slice(0, -1)) // Remove loading message
        });

        const data = await response.json();

        // Remove loading message and add real response
        chat_messages.pop(); // Remove loading message
        const response_message = {
            role: 'assistant', 
            content: data.status === 200 
                ? data.data 
                : 'Sorry, I encountered an error. Please try again.'
        };
        chat_messages.push(response_message);
        await chrome.storage.local.set({ wiz_chat: chat_messages });

    } catch (error) {
        console.error('Error sending message:', error);
        // Remove loading message if it exists
        if (chat_messages[chat_messages.length - 1].content === 'Thinking...') {
            chat_messages.pop();
        }
        chat_messages.push({ 
            role: 'assistant', 
            content: 'Sorry, I encountered an error. Please try again.' 
        });
        await chrome.storage.local.set({ wiz_chat: chat_messages });
    } finally {
        is_sending_message = false;
        loadChatMessages();
    }
}

// Load the messages
function loadChatMessages() {
    const base_styles = `
        padding: 10px;
        border-radius: 4px;
        line-height: 1.25;
        max-width: 80%;
        list-style-type: disc;
        font-size: 14px;
        margin-bottom: 13px !important;
    `;
    const role_styles = {
        assistant: `background-color: #152D2E; border: 1px solid #00E6D2; color: white; align-self: flex-start;`,
        user: `background-color: #1A2023; border: 1px solid #313131; color: white; align-self: flex-end;`
    };
    chrome.storage.local.get('wiz_chat', function(result) {
        const messages_container = document.getElementById("wiz-chat-messages-container");
        messages_container.innerHTML = '';
        (result.wiz_chat || []).forEach(message => {
            const message_element = document.createElement('div');
            message_element.style.cssText = base_styles + (role_styles[message.role] || role_styles.user);
            message_element.classList.add('wiz_message');
            // message_element.innerHTML = message.content;
            // Basic markdown handling
            let content = message.content
                // Handle code blocks
                .replace(/```(.*?)\n([\s\S]*?)```/g, '<pre style="background-color: white; padding: 10px; border-radius: 4px; overflow-x: auto; border: 1px solid #00E6D2;"><code>$2</code></pre>')
                // Handle inline code
                .replace(/`([^`]+)`/g, '<code style="background-color: white; padding: 2px 4px; border-radius: 2px; border: 1px solid #00E6D2;">$1</code>')
                // Handle bold
                .replace(/\*\*(.*?)\*\*/g, '<strong>$1</strong>')
                // Handle bold with underscores
                .replace(/\b_(.*?)_\b/g, '<strong>$1</strong>')
                // Handle italic
                .replace(/\*(.*?)\*/g, '<em>$1</em>') 
                // Handle italic with underscores
                .replace(/_(.*?)_/g, '<em>$1</em>')
                // Handle underline
                .replace(/__(.*?)__/g, '<u>$1</u>')
                // Handle strikethrough
                .replace(/~~(.*?)~~/g, '<del>$1</del>')
                // Handle links
                .replace(/\[(.*?)\]\((.*?)\)/g, '<a href="$2" target="_blank" style="color: white; text-decoration: underline;">$1</a>')
                // Handle headers as paragraphs
                .replace(/^#+\s+(.*)$/gm, '<p>$1</p>');

            // Handle lists by first identifying list sections
            const lines = content.split('\n');
            let inList = false;
            let listContent = '';
            let processedContent = '';

            lines.forEach(line => {
                // Check if line is a list item
                const listMatch = line.match(/^\s*[-*+]\s+(.*)$/) || line.match(/^\s*\d+\.\s+(.*)$/);
                if (listMatch) {
                    if (!inList) {
                        inList = true;
                        listContent = '<ul>';
                    }
                    listContent += `<li>${listMatch[1]}</li>`;
                } else {
                    if (inList) {
                        inList = false;
                        listContent += '</ul>';
                        processedContent += listContent;
                        listContent = '';
                    }
                    processedContent += line + '\n';
                }
            });

            // Handle any remaining list content
            if (inList) {
                processedContent += listContent + '</ul>';
            }

            message_element.innerHTML = processedContent;
            messages_container.appendChild(message_element);
        });
        document.getElementById("wiz-chat-messages-scroll-container").scrollTop = document.getElementById("wiz-chat-messages-scroll-container").scrollHeight;
    });
}
loadChatMessages();

// Send message on button click
document.querySelector('[wizdata-button="send_query"]').addEventListener('click', sendChatMessage);

// Send message on Enter key
wiz_chatinput.addEventListener('keypress', function(event) {
    if (event.key === 'Enter') {
        event.preventDefault();
        sendChatMessage();
    }
});

// Reset sending flag when input changes
wiz_chatinput.addEventListener('input', function() {
    is_sending_message = false;
});

document.querySelector('[wizdata-button="chat"]').addEventListener('click', () => {
    chrome.storage.local.get(['wiz_settings']).then(result => {
        chrome.storage.local.set({ 
            wiz_settings: {
                ...result.wiz_settings,
                chat_visible: !result.wiz_settings.chat_visible,
                history_visible: false
            }
        });
    });
});

document.querySelector('[wizdata-button="clear_wizchat"]').addEventListener('click', function() {
    chrome.storage.local.set({ wiz_chat: [{ role: 'assistant', content: 'Hi! I\'m WizChat, I\'m here help your to answer any of your questions!' }] }, function() {
        loadChatMessages();
    });
});
